% In this computational exploration we want to integrate the logistic
% equation numerically. We will do this following two approaches. First, we
% will write our own code for numerical integration. Second, as a way of
% introducing Matlab's capabilities. We will use built-in functions in
% order to accomplish the same task.

% Let's start by defining the parameters of the equation.
r=1;   % Rate constant in the logistic equation
K=1;   % Saturation population size

% If we look at equation 3.12, which is the basis of this numerical
% integration, we see that we need to choose a size for the time interval
% "Delta t". This time step should be much smaller than the smallest time
% scale in the problem. The only time scale in this equation is given by
% the parameter "r". As a result we can safely choose
step=0.001;


% We also need to decide how long to integrate for. This total integration
% time is
TotalTime=15;

% Finally, we need to choose a starting condition
N0=0.0001;


% We are now ready to perform the numerical integration. We will do two
% such integrations in parallel. For exponential growth we will record the
% number as a function of time in the vector "Ne". For the logistic
% equation we will do so in the vector "N". We then set the first element
% of each of these vectors to our initial condition "N0" specified above
N(1)=N0;
Ne(1)=N0;

% In order to calculate the size of the population at a given time point
% we will use the population size at the previous time point. As a 
% result we need to move through
% time using a for-loop. Notice that this for loop starts at time point
% "step" and moves to "TotalTime" using a stepping given by "step". We
% don't start from zero because we have already determined the initial
% conditions in "N(1)" and "Ne(1)". At the same time that we move forward
% in time using the for-loop we need a counter that will relate the time
% point to the position along the vectors "N" and "Ne" where the
% corresponding information was stored. We do this using the counter "m",
% which we initialize as
m=2;

%Now we are ready for the loop. Notice how after each iteration we increase
%the value of the counter "m", which determines a new position in the
%vectors "N" and "Ne".
for i=step:step:TotalTime
    % Calculate the number at time point "m" given the numbers at time
    % point "m-1" and the parameters of the problem.
    N(m)=N(m-1)+(r/K)*step*N(m-1)*(K-N(m-1));   % Logistic equation
    Ne(m)=Ne(m-1)+r*step*Ne(m-1);               % Exponential growth
    m=m+1;                                      % Increment the counter
end

% Before we plot this we want to create a vector whose elements correspond
% to the time points in each element of "N" and "Ne". That way we will be
% able to plot number as a function of time.
Times=0:step:TotalTime;  

%How we are ready to make the plot. We will compare both outcomes.
plot(Times,N,'-b');
hold on
plot(Times, Ne,'-r');
hold off
ylim([0,1])
xlabel('time')
ylabel('number of cells')
legend('logistic equation','exponential growth','Location','SouthEast')



% Finally, let's solve the logistic equation using one of Matlab's built-in
% functions. Matlab has many different solvers. We choose to use ode45, but
% the reader is encouraged to check the Matlab documentation to appreciate
% the differences between the algorithms employed by each of these
% functions.

% Before we invoke this function we need to define a function that given a
% number it gives us the derivate. Basically, we need a function that
% returns the right-hand side of equation 3.8. This can be done with the
% somehow confusing syntax
f=@(t,x) r*x*(1-x/K);

% "f" is the function we just defined. Using the command "@" we told Matlab
% that it is a function of both "t" and "x". Although equation 3.8 does not
% depend explicitly on time it is necessary to define it here so that "f"
% will work with the numerical integration functions from Matlab.

%Note that "f" is just a function so that we can do things like
f(0,0.5)

% Now we are ready to do the actual numerical integration using the command
% "ode45". We will specify "f", the initial (0) and final time points
% ("TotalTime") as well as the initial condition, "N0". Notice that "ode45"
% does not need to have the time interval "step" specified. It figures out
% what the best time interval is
[Timeode45,Node45]=ode45(f,[0,TotalTime],N0);

% Now we can compare the results of our numerical integration and Matlab's.
plot(Times,N,'-b');
hold on
plot(Timeode45, Node45,'-r');
hold off
ylim([0,1])
xlabel('time')
ylabel('number of cells')
legend('our integration','Matlab''s integration','Location','SouthEast')


% We see tha that the agreement is really good. The reader is encouraged to
% try to find conditions where our numerical integration fails with respect
% to Matlab's. For example, what would have happened if we had chosen a
% much larger step?


